#include "cproj.h"

/* Variables common to all subroutines in this code file
  -----------------------------------------------------*/
static double lon_center;       /* Center longitude (projection center) */
static double R;                /* Radius of the earth (sphere) */
static double false_easting;    /* x offset in meters                   */
static double false_northing;   /* y offset in meters                   */

/* Initialize the Aitoff projection
  ------------------------------------*/
aitint(r, center_long, false_east, false_north) 
double r;                       /* (I) Radius of the earth (sphere) */
double center_long;             /* (I) Center longitude */
double false_east;              /* x offset in meters                   */
double false_north;             /* y offset in meters                   */
{
/* Place parameters in static storage for common use
  -------------------------------------------------*/
false_easting = false_east;
false_northing = false_north;
R = r;
lon_center = center_long;

return(OK);
}


/* Aitoff forward equations--mapping lat,long to x,y
  ----------------------------------------------------*/
aitfor(lon, lat, x, y)
double lon;                     /* (I) Longitude */
double lat;                     /* (I) Latitude */
double *x;                      /* (O) X projection coordinate */
double *y;                      /* (O) Y projection coordinate */
{
double cos_lat;
double theta;
double cos_theta;
double sin_theta;

/* Forward equations
  -----------------*/
cos_lat = cos(lat);
cos_theta = cos_lat * cos(lon / 2);
sin_theta = sqrt(1 - cos_theta * cos_theta);
theta = acos(cos_theta);
*x = 2 * R * theta * cos_lat * sin(lon / 2) / sin_theta;
*y = R * theta * sin(lat) / sin_theta;
return OK;
}

/* Aitoff inverse equations--mapping x,y to lat,long 
  ----------------------------------------------------*/

aitinv(x, y, lon, lat)
double x;                       /* (O) X projection coordinate */
double y;                       /* (O) Y projection coordinate */
double *lon;                    /* (I) Longitude */
double *lat;                    /* (I) Latitude */
{
int i;
double lat1, lat2, cos_lat, sin_lat, cos_theta, f1, f2, delta_lat;

double fac, tmp;

fac = sqrt(4.0 * R * R - (x * x)/ 4.0 - y * y) / 2.0;
tmp = 2.0 * R * R - x * x/4 - y * y;
if (tmp < 0) return IN_BREAK;
*lon = adjust_lon(lon_center + 2.0 * atan2(x * fac, tmp));
*lat = asinz(y * fac / R / R);
return OK;

if (y == 0) {
  *lat = 0;
  *lon = x / R;
  return OK;
}

if (x == 0) {
  *lat = y / R;
  *lon = 0;
  return OK;
}

/*
lat1 = y / R;
*lon = 2 * asin(x * tan(lat1) / 2 / y);
cos_theta = cos(lat1) * cos(*lon / 2);
f1 = y * sqrt(1 - cos_theta * cos_theta) - R * acos(cos_theta) * sin(lat1);
*/
lat1 = y / R;
f1 = (y - R * lat1) * sin(lat1);
lat2 = lat1 / 1.5;

/* Iterate using the Newton-Raphson method to find theta
  -----------------------------------------------------*/
   for (i=0;;i++)
      {
      cos_lat = cos(lat2);
      sin_lat = sin(lat2);
      *lon = 2 * asin(x * sin_lat / cos_lat / 2 / y);
      cos_theta = cos_lat * cos(*lon / 2);
      f2 = y * sqrt(1 - cos_theta * cos_theta) - R * acos(cos_theta) * sin_lat;
      delta_lat = - f2 / (f2 - f1) * (lat2 - lat1);
      lat2 += delta_lat;
      if (fabs(delta_lat) < EPSLN) break;
if (i > 20) break;
/*
      if (i >= 50) 
         {
         p_error("Iteration failed to converge","Aitoff-inverse");
         return(251);
         }
*/
      lat1 = lat2;
      f1 = f2;
      }

*lat = lat2;
*lon = 2 * asin(x * tan(*lat) / 2 / y);

return OK;
}
