#include "cproj.h"

static double R;
static double ratio[2];
static double scale[2];
static double lon_center;
static double trunc_lat;
static double angular_step[2];
static double false_easting;
static double false_northing;
static double lon_step;
static double *cosalpha[2];
static double *sinalpha[2];


int collint(double r, double center_long, double false_east, double false_north,
            int Nzones, double total_angle, double truncature_lat)
{
int lon_region, lat_region;

    R = r;
    lon_center = center_long;
    false_easting = false_east;
    false_northing = false_north;
    trunc_lat = truncature_lat;
    lon_step = 360. / Nzones;
    angular_step[0] = D2R * total_angle / Nzones;
    angular_step[1] = D2R * lon_step;
    for (lat_region=0; lat_region<2; lat_region++) {
      ratio[lat_region] = tan(D2R * lon_step / 2) / tan(angular_step[lat_region] / 2);
      scale[lat_region] = cos(angular_step[lat_region] / 2) * sin(angular_step[0] / 2) / sin(angular_step[lat_region] / 2);
      cosalpha[lat_region] = (double *) malloc(Nzones * sizeof(double));
      sinalpha[lat_region] = (double *) malloc(Nzones * sizeof(double));
      for (lon_region=0; lon_region<Nzones; lon_region++) {
        cosalpha[lat_region][lon_region] = cos(HALF_PI + angular_step[lat_region] * (lon_region - 1));
        sinalpha[lat_region][lon_region] = sin(HALF_PI + angular_step[lat_region] * (lon_region - 1));
      }
    }
    return(OK);
}



int collfor(double lon, double lat, double *xproj, double *yproj)
{
int lon_region, lat_region;
double xx, yy;

    lon -= lon_center;
    if (lon >  180) lon -= 360;
    if (lon < -180) lon += 360;
    if (fabs(lat) > 90) return(ERROR);
    lon_region = (180 + lon) / lon_step;
    if (fabs(lat) < trunc_lat) lat_region = 0;
                   else lat_region = 1;
    xx = (90 - fabs(lat)) * tan((lon + 180 - lon_step * (lon_region + 0.5)) * D2R) / ratio[lat_region] * scale[lat_region];
    yy = (90 - fabs(lat)) * scale[lat_region];
    *yproj =  xx * sinalpha[lat_region][lon_region] - yy * cosalpha[lat_region][lon_region];
    if (lat >= 0) *xproj =  xx * cosalpha[lat_region][lon_region] + yy * sinalpha[lat_region][lon_region];
             else *xproj = -xx * cosalpha[lat_region][lon_region] - yy * sinalpha[lat_region][lon_region];
    *yproj += false_northing;
    if (lat >= 0) *xproj += false_easting - R * scale[0] + (90 - trunc_lat) * (scale[0] - scale[lat_region]);
             else *xproj += false_easting + R * scale[0] - (90 - trunc_lat) * (scale[0] - scale[lat_region]);
    return(OK);

}



int collinv(double xproj, double yproj, double *lon, double *lat)
{
int lon_region, lat_region;
double xx, yy, theta;

    xproj -= false_easting;
    yproj -= false_northing;
    for (lat_region=0; lat_region<2; lat_region++) {
        xx = R * scale[0] - fabs(xproj) - (90 - trunc_lat) * (scale[0] - scale[lat_region]);
        yy = yproj;
        if (lat_region == 0  && xx * xx + yy * yy < (90 - trunc_lat) * (90 - trunc_lat) * scale[0] * scale[0]) continue;
        if (lat_region == 1  && xx * xx + yy * yy > (90 - trunc_lat) * (90 - trunc_lat)) return(IN_BREAK);
        theta = atan2(yy, xx);
        if (theta < -1.5 * angular_step[lat_region]) theta += TWO_PI;
        lon_region = theta / angular_step[lat_region] + 1.5;
        if (lon_region >= (int) (360 / lon_step)) continue;
        *lon = (-xx * cosalpha[lat_region][lon_region] - yy * sinalpha[lat_region][lon_region]) / scale[lat_region];
        if (xproj <= 0)
          *lat = ( xx * sinalpha[lat_region][lon_region] - yy * cosalpha[lat_region][lon_region]) / scale[lat_region];
        else
          *lat = (-xx * sinalpha[lat_region][lon_region] + yy * cosalpha[lat_region][lon_region]) / scale[lat_region];
        if (fabs(*lat) > 90) return(IN_BREAK);
        if (lat_region == 1  &&  fabs(*lat) > 90 - trunc_lat) return(IN_BREAK);
        *lon = -atan2(*lon * ratio[lat_region], fabs(*lat)) * R2D;
        if (90 - fabs(*lat) < trunc_lat) break;
    }
    if (fabs(*lon) > lon_step / 2) return(IN_BREAK);
    *lon += lon_center - 180 + lon_step * (lon_region + 0.5);
    if (*lon >  180) *lon -= 360;
    if (*lon < -180) *lon += 360;
    if (xproj <= 0) *lat =  90 - *lat;
               else *lat = -90 - *lat;
    return(OK);

}
