#include "cproj.h"

#define MAXZONES 86400

static double R;
static double lon_center;
static double false_easting;
static double false_northing;
static double lat_step;
static double xratio[MAXZONES];
static double offset[MAXZONES];
static int Nzones;


int isinusint(double r, double center_long, double false_east, double false_north, int nzones)
{
int izone, ncol;

R = r;
Nzones = nzones;
lon_center = center_long;
false_easting = false_east;
false_northing = false_north;
lat_step = 180. / nzones;

if (nzones % 2 != 0) {
  fprintf(stderr, "Number of zones (%d) must be even in isinusint().\n", nzones);
  exit(-1);
}
if (nzones > MAXZONES) {
  fprintf(stderr, "Number of zones (%d) is greater than %d in isinusint().\n", nzones, MAXZONES);
  exit(-1);
}

for (izone=0; izone<nzones/2; izone++) {
  ncol = 2 * cos(HALF_PI * (1 - (izone + 0.5) / (nzones / 2))) * nzones + 0.5;
  if (ncol < 1) ncol = 1;
  xratio[izone] = R * ncol / (2 * nzones);
  if (ncol % 2 == 1) offset[izone] = 0.5 * R * 180. / nzones;
               else  offset[izone] = 0;
  offset[nzones - 1 - izone] = offset[izone];
  xratio[nzones - 1 - izone] = xratio[izone];
}

return OK;

}




int isinusfor(double lon, double lat, double *x, double *y)
{
int izone;

if (fabs(lat) > 90) return ERROR;
izone = (90 - lat) / lat_step;
*x = lon * xratio[izone];
*x -= offset[izone];
*y = R * lat;
return OK;
}



int isinusinv(double x, double y, double *lon, double *lat, int aggfactor)
{
int izone;
int izone1, izone2, j;

*lat = y / R;
if (fabs(*lat) >= 90) return ERROR;
if (aggfactor == 1) {
    izone = (90 - *lat) / lat_step;
    x += offset[izone];
    *lon = x / xratio[izone];
    if (fabs(*lon) > 180) return ERROR;
} else {
    if (Nzones % aggfactor != 0) {
      fprintf(stderr, "Number of zones (%d) must be divisible by aggregation factor (%d) in isinusinv().\n", Nzones, aggfactor);
      exit(-1);
    }
    izone1 = (90 - *lat) / lat_step - (aggfactor - 1) / 2.;
    izone2 = izone1 + aggfactor - 1;
    *lon = 0;
    for (izone=izone1; izone<=izone2; izone++)
      for (j=0; j<aggfactor; j++)
        *lon += (x + offset[izone] + j - (aggfactor - 1) / 2.) / xratio[izone];
    *lon /= aggfactor * aggfactor;
    if (fabs(*lon) > 180) return ERROR;
}
return OK;
}
